<?php

/**
 * @copyright   Copyright (C) 2009 - 2017 Michael RIchey. All rights reserved.
 * @license             GNU General Public License version 2 or later; see LICENSE.txt
 */
// No direct access
defined('_JEXEC') or die;

class plgSystemHeadTag extends JPlugin {

    var $_cache = null;
    var $_doc = null;
    private $_afterrender = null;
    private $_localonly = null;
    private $_host = null;
    private $_user = null;
    protected $app;

    public function __construct($subject, $config) {
	parent::__construct($subject, $config);
	$this->_doc = JFactory::getDocument();
	$this->_afterrender = $this->params->get('afterrender', 0);
	if($this->_afterrender) {
	    require_once(__DIR__ . '/tagparser.php');
	}
	$this->_localonly = $this->params->get('localonly', false);
	$this->_user = JFactory::getUser();
	if ($this->_localonly)
	{
	    $this->_host = JURI::getInstance()->getHost();
	}
    }

    function onBeforeRender() {
	if ($this->app->isAdmin() || $this->_doc->getType() != 'html') {
	    return true;
	}
	$menu = $this->app->getMenu();
	$activemenu = $menu->getActive();
	$itemid = is_object($activemenu) ? $activemenu->id : 0;
	$accesslevel = (array) $this->_user->getAuthorisedViewLevels();
	$this->_usergroup = (array) $this->_user->getAuthorisedGroups();
	foreach ((array) $this->params->get('menuitem') as $rule)
	{
	    if (in_array($itemid, $rule->menuitem))
	    {
		$this->_headtag($rule);
	    }
	}
	foreach ((array) $this->params->get('usergroup') as $rule)
	{
	    if (array_intersect($this->_user->groups, $rule->usergroup))
	    {
		$this->_headtag($rule);
	    }
	}
	foreach ((array) $this->params->get('accesslevel') as $rule)
	{
	    if (array_intersect($this->_user->getAuthorisedViewLevels(), $rule->accesslevel))
	    {
		$this->_headtag($rule);
	    }
	}
    }

    function onContentPrepare($context, &$article, &$params, $page = 0) {
	if ($this->app->isAdmin() || !$this->_afterrender || $this->_doc->getType() != 'html')
	    return true;
	$article->text = $this->_scanContent($article->text);
    }

    function onAfterRender() {
	if ($this->app->isAdmin() || !$this->_afterrender || $this->_doc->getType() != 'html')
	    return true;
	$body = $this->_scanContent(JResponse::getBody(), true);
	JResponse::setBody($body);
    }

    function _scanContent($body, $afterrender = false) {
	$accesslevel = (array) $this->_user->getAuthorisedViewLevels();
	$this->_usergroup = (array) $this->_user->getAuthorisedGroups();
	if (strpos($body, '{headtag:') === false)
	    return $body; // only operate on pages that contain the headtag code
	$tagparser = plg_system_headtag\TagParser::getInstance('headtag', array('alternateclose' => 'headtag'));
	$tagarray = array('scriptdeclaration', 'script', 'styledeclaration', 'stylesheet', 'customtag');
	foreach ($tagarray as $type)
	{
	    $tagparser->setIdentifier('headtag:' . $type);
	    $tagparser->tags = array();
	    $tagparser->parse($body);
	    foreach ($tagparser->tags as $tag)
	    {
		$accesslevels = $tag->getParam('accesslevel', false);
		$this->_usergroups = $tag->getParam('usergroup', false);
		$exit = false;
		if ($accesslevels || $this->_usergroups)
		{
		    $exit = true;
		    if (
			    ($accesslevels && array_intersect($accesslevel, explode(',', $accesslevels))) ||
			    ($this->_usergroups && array_intersect($this->_usergroup, explode(',', $this->_usergroups)))
		    )
		    {
			$exit = false;
		    }
		}
		if (!$exit)
		{
		    $rule = new stdClass();
		    $rule->type = $type;
		    $rule->script_stylesheet = $tag->content;
		    $rule->declaration_custom = $tag->content;
		    $this->_headtag($rule);
		}
		$body = $tag->replace($body, '');
	    }
	}
	return $body;
    }

    private function _headtag($rule) {
	switch ($rule->type)
	{
	    case 'script':
		$this->_addScript($rule->script_stylesheet);
		break;
	    case 'scriptdeclaration':
		$this->_doc->addScriptDeclaration($rule->declaration_custom);
		break;
	    case 'stylesheet':
		$this->_addStyleSheet($rule->script_stylesheet);
		break;
	    case 'styledeclaration':
		$this->_doc->addStyleDeclaration($rule->declaration_custom);
		break;
	    case 'customtag':
		$this->_doc->addCustomTag($rule->declaration_custom);
		break;
	}
    }

    private function _addScript($link) {
	if ($this->_localonly && !$this->_verifyLocal($link))
	{
	    return;
	}
	$this->_doc->addScript($link);
    }

    private function _addStyleSheet($link) {
	if ($this->_localonly && !$this->_verifyLocal($link))
	{
	    return;
	}
	$this->_doc->addStyleSheet($link);
    }

    private function _verifyLocal($link) {
	$url = parse_url($link);
	if (isset($url['host']) && $url['host'] != $this->_host)
	{
	    return false;
	}
	return true;
    }

}
